#include <avr/io.h>		// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support
#include <util/delay.h>
#include "global.h"

u08 volatile message[21];
u08 volatile nibble;
u16 volatile period;
u08 volatile nibble_cntr = 0;
u08 volatile lastbit;
u16 volatile ICR1tmp = 0;
u08 volatile preamble;
u08 status;

enum global_state
{
	IDLE = 0,
	PREAMBLE_DETECTED = 1,
	SYNC_AWAITING = 2,
	SYNC_RECEIVING = 3,
	DATA_RECEIVING = 4
};

u08 volatile bit_cntr;
u08 volatile bit_cntr2;
u08 volatile preamble_cntr;
u08 volatile manchester[3];
 char volatile STATE_DECODER = IDLE;;


void USART_Init( unsigned int ubrr_value)
{
/*Set baud rate */
UBRR0H = (unsigned char)(ubrr_value>>8);
UBRR0L = (unsigned char)ubrr_value;
/*Enable receiver and transmitter */
UCSR0B = (1<<RXEN0)|(1<<TXEN0);
/* Set frame format: 8data, 2stop bit */
UCSR0C = 0b110;
}

void USARTWriteChar(char data)
{
/* Wait for empty transmit buffer */
while ( !( UCSR0A & (1<<UDRE0)) )
;
/* Put data into buffer, sends the data */
UDR0 = data;
}

char USARTReadChar()
{
/* Wait for data to be received */
while ( !(UCSR0A & (1<<RXC0)) )
;
/* Get and return received data from buffer */
return UDR0;
}

void USARTWriteString(char *string)
{
	
	while (*string)
	USARTWriteChar(*string++);
}


int main(void)
{
TCCR1B = 0b11000001;
TIMSK1 = 0b00100000;
EICRA = 0b00000001; /*external INT0 */
EIMSK = 1;
DDRB = 0b00000001;
PORTB = 0;
PORTD = 0b00000011;



USART_Init(26); 



USARTWriteString("Stacja pogodowa  \n \r");


sei();



while(1);


}

ISR(TIMER1_CAPT_vect )

{
PORTB &= 0b11111110;
period =   ICR1 - ICR1tmp;
ICR1tmp = ICR1;
	
	if(period > 800 && period < 1200)
	{	
		if (STATE_DECODER == IDLE ) preamble_cntr++;
		if (preamble_cntr>17 && STATE_DECODER == IDLE) {STATE_DECODER = SYNC_AWAITING; }

		if (STATE_DECODER == SYNC_RECEIVING || STATE_DECODER == DATA_RECEIVING)
		{
		bit_cntr+= 2;
		if(bit_cntr == 4)
			bit_received();
			if (bit_cntr2 == 4)
			nibble_received();
		}
	}
	else if(period > 330 && period < 600)
	{	
	if (STATE_DECODER == SYNC_AWAITING) STATE_DECODER = SYNC_RECEIVING;

	if (STATE_DECODER == SYNC_RECEIVING || STATE_DECODER == DATA_RECEIVING)
	{
	bit_cntr++;
		if(bit_cntr == 4)
		{
			return_to_idle();
		}
	}
	}

	else 
		return_to_idle();
	
}

ISR(INT0_vect )
{
//USARTWriteChar('A');
	if bit_is_set(PIND, 2) lastbit = 1;
	else lastbit = 0;
	PORTB |= 0b00000001;

}

void return_to_idle()
{
	STATE_DECODER = IDLE;
	preamble_cntr = 0;
	bit_cntr = 0;
	bit_cntr2 = 0;
	nibble_cntr = 0;
	nibble = 0;
}

bit_received()
{
		if(lastbit) 
		{
//		USARTWriteChar('0');
		}
		else 
		{
//		USARTWriteChar('1'); 
		nibble |= 0b10000000;
		}
		nibble = nibble>>1;
		bit_cntr = 0;
		bit_cntr2++;
}
	
nibble_received()
{
	bit_cntr2 = 0;
//	USARTWriteChar('0'+ nibble_cntr);
	nibble = nibble>>3;
		if (STATE_DECODER == SYNC_RECEIVING)
		{
			if (nibble != 0x0A) return_to_idle();
			else 
			STATE_DECODER = DATA_RECEIVING;
			
		}


		if (STATE_DECODER == DATA_RECEIVING)
		{
		message[nibble_cntr] = nibble;
		nibble_cntr++;
		if (nibble_cntr == 20) {send_temp();return_to_idle();}
		}
		
}

send_temp()
{
cli();
	u08 checksum = 0;
	for (u08 i=1; i<16; i++) checksum += message[i];

	if (checksum == ((message[17]*16) + message[16]))
	{

	if (message[12]==0) USARTWriteChar('+');
	else USARTWriteChar('-');

	USARTWriteChar('0' + message[11]);
	USARTWriteChar('0' + message[10]);
	USARTWriteChar(',');
	USARTWriteChar('0' + message[9]);
	USARTWriteString("st. C ");
	USARTWriteChar('0' + message[14]);
	USARTWriteChar('0' + message[13]);
	USARTWriteString("%  ");
		USARTWriteChar('0' + message[18]);
	USARTWriteChar('0' + message[19]);
	USARTWriteChar(10);
	USARTWriteChar(13);
	}
	else 
	{USARTWriteString("Blad sumy kontrolnej");
		USARTWriteChar(10);
	USARTWriteChar(13);}
	

sei();
}
